// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "TextureAsset.hpp"

using namespace Dawn;

TextureAsset::TextureAsset(AssetManager *assetManager, std::string name) :
  Asset(assetManager, name),
  loader(name + ".texture"),
  texture()
{
}

void TextureAsset::updateSync() {
  if(
    this->state != 0x03
  ) return;

  this->state = 0x04;
  this->texture.setSize(this->width, this->height);
  this->texture.buffer(this->colors);
  this->state = 0x05;
  this->loaded = true;
}

void TextureAsset::updateAsync() {
  if(this->state != 0x00) return;
  this->state = 0x01;
  this->loader.loadRaw(&this->buffer);
  this->state = 0x02;

  // Parse header data.
  char integer[32];
  size_t j = 0, i = 0;
  while(true) {
    auto c = this->buffer[i++];
    if(c == '|') {
      integer[j] = '\0';
      if(this->width == -1) {
        this->width = atoi(integer);
        assertTrue(this->width > 0);
        j = 0;
        continue;
      } else {
        this->height = atoi(integer);
        assertTrue(this->height > 0);
        break;
      }
    }
    integer[j++] = c;
  }

  this->colors = (struct Color *)((void *)(this->buffer + i));
  this->state = 0x03;
}

TextureAsset::~TextureAsset() {
  if(this->buffer != nullptr) {
    memoryFree(this->buffer);
  }
}